#!/usr/bin/env python

# Project skeleton maintained at https://github.com/jaraco/skeleton

import setuptools

params = dict(
	use_scm_version=True,
)
if __name__ == '__main__':
	setuptools.setup(**params)
